@@ -1,12 +1,12 @@
 ;;; deadgrep.el --- fast, friendly searching with ripgrep  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2018  Wilfred Hughes
+;; Copyright (C) 2018-2024  Wilfred Hughes
 
 ;; Author: Wilfred Hughes <me@wilfred.me.uk>
 ;; URL: https://github.com/Wilfred/deadgrep
 ;; Keywords: tools
-;; Package-Version: 0.13
-;; Package-Revision: ad7ddfbce9d1
+;; Package-Version: 20241210.1630
+;; Package-Revision: bb555790c6f4
 ;; Package-Requires: ((emacs "25.1") (dash "2.12.0") (s "1.11.0") (spinner "1.7.3"))
 
 ;; This program is free software; you can redistribute it and/or modify
@@ -150,6 +150,13 @@ display."
 (defvar-local deadgrep--file-type 'all)
 (put 'deadgrep--file-type 'permanent-local t)
 
+(defvar-local deadgrep--skip-if-hidden nil
+  "Whether deadgrep should ignore hidden files (e.g. files called .foo).")
+(put 'deadgrep--skip-if-hidden 'permanent-local t)
+(defvar-local deadgrep--skip-if-vcs-ignore 't
+  "Whether deadgrep should ignore files if they're listed in .gitignore.")
+(put 'deadgrep--skip-if-vcs-ignore 'permanent-local t)
+
 (defvar-local deadgrep--context nil
   "When set, also show context of results.
 This is stored as a cons cell of integers (lines-before . lines-after).")
@@ -498,6 +505,24 @@ with a text face property `deadgrep-match-face'."
   'case nil
   'help-echo "Change file type")
 
+(define-button-type 'deadgrep-skip-hidden-type
+  'action #'deadgrep--skip-if-hidden
+  'case nil
+  'help-echo "Toggle whether to skip dotfiles")
+
+(defun deadgrep--skip-if-hidden (_button)
+  (setq deadgrep--skip-if-hidden (not deadgrep--skip-if-hidden))
+  (deadgrep-restart))
+
+(define-button-type 'deadgrep-vcs-skip-type
+  'action #'deadgrep--skip-if-vcs-ignore
+  'case nil
+  'help-echo "Toggle whether to skip files listed in .gitignore")
+
+(defun deadgrep--skip-if-vcs-ignore (_button)
+  (setq deadgrep--skip-if-vcs-ignore (not deadgrep--skip-if-vcs-ignore))
+  (deadgrep-restart))
+
 (defun deadgrep--format-file-type (file-type extensions)
   (let* ((max-exts 4)
          (truncated (> (length extensions) max-exts)))
@@ -737,6 +762,17 @@ to obtain ripgrep results."
       (push (format "--before-context=%s" (car context)) args)
       (push (format "--after-context=%s" (cdr context)) args))
 
+    (unless deadgrep--skip-if-hidden
+      (push "--hidden" args))
+    (if deadgrep--skip-if-vcs-ignore
+        ;; By default, ripgrep searches .git even when it's respecting
+        ;; .gitignore, if --hidden is set. Ignore .git when we're
+        ;; using .gitignore.
+        ;;
+        ;; https://github.com/BurntSushi/ripgrep/issues/713
+        (push "--glob=!/.git" args)
+      (push "--no-ignore-vcs" args))
+
     (push "--" args)
     (push search-term args)
     (push "." args)
@@ -840,6 +876,14 @@ search settings."
             (if (eq (car-safe deadgrep--file-type) 'glob)
                 (format ":%s" (cdr deadgrep--file-type))
               "")
+            "\n"
+            (propertize "Skip: "
+                        'face 'deadgrep-meta-face)
+            (deadgrep--button "dotfiles" 'deadgrep-skip-hidden-type)
+            (if deadgrep--skip-if-hidden ":yes" ":no")
+            " "
+            (deadgrep--button ".gitignore items" 'deadgrep-vcs-skip-type)
+            (if deadgrep--skip-if-vcs-ignore ":yes" ":no")
             "\n\n")
     (put-text-property
      start-pos (point)
@@ -1225,7 +1269,6 @@ If POS is nil, use the beginning position of the current line."
       ;; consistent with `compilation-next-error-function' and also
       ;; useful with `deadgrep-visit-result-other-window'.
       (setq overlay-arrow-position (copy-marker pos))
-      (setq next-error-last-buffer (current-buffer))
 
       (funcall open-fn file-name)
       (goto-char (point-min))
@@ -1536,8 +1579,6 @@ for a string, offering the current word as a default."
       (let* ((sym (symbol-at-point))
              (sym-name (when sym
                          (substring-no-properties (symbol-name sym))))
-             ;; TODO: prompt should say search string or search regexp
-             ;; as appropriate.
              (prompt
               (deadgrep--search-prompt sym-name)))
         (setq search-term
@@ -1628,10 +1669,13 @@ deadgrep is ready but not yet searching."
                (key-description restart-key))))))
 
 (defun deadgrep--mode-line ()
-  (let ((s (if deadgrep--result-count
-               (format "Deadgrep:%s" deadgrep--result-count)
-             "Deadgrep")))
-    (concat s (spinner-print deadgrep--spinner))))
+  (let* ((s (if deadgrep--result-count
+                (format "Deadgrep:%s" deadgrep--result-count)
+              "Deadgrep"))
+         (spinner-str (spinner-print deadgrep--spinner)))
+    (if spinner-str
+        (concat s " " spinner-str)
+      s)))
 
 (defun deadgrep--create-imenu-index ()
   "Create `imenu' index for matched files."
@@ -1659,24 +1703,31 @@ don't actually start the search."
                    (buffer-file-name))))
          (last-results-buf (car-safe (deadgrep--buffers)))
          prev-search-type
-         prev-search-case)
+         prev-search-case
+         prev-skip-if-hidden
+         prev-skip-if-vcs-ignore)
     ;; Find out what search settings were used last time.
     (when last-results-buf
       (with-current-buffer last-results-buf
         (setq prev-search-type deadgrep--search-type)
-        (setq prev-search-case deadgrep--search-case)))
+        (setq prev-search-case deadgrep--search-case)
+        (setq prev-skip-if-hidden deadgrep--skip-if-hidden)
+        (setq prev-skip-if-vcs-ignore deadgrep--skip-if-vcs-ignore)))
 
     (funcall deadgrep-display-buffer-function buf)
 
     (with-current-buffer buf
       (setq imenu-create-index-function #'deadgrep--create-imenu-index)
       (setq next-error-function #'deadgrep-next-error)
+      (setq next-error-last-buffer buf)
 
       ;; If we have previous search settings, apply them to our new
       ;; search results buffer.
       (when last-results-buf
         (setq deadgrep--search-type prev-search-type)
-        (setq deadgrep--search-case prev-search-case))
+        (setq deadgrep--search-case prev-search-case)
+        (setq deadgrep--skip-if-hidden prev-skip-if-hidden)
+        (setq deadgrep--skip-if-vcs-ignore prev-skip-if-vcs-ignore))
 
       (deadgrep--write-heading)
 
